@@ -173,8 +173,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 				* esScale * esScale * plist->parm_scale
 				* 1E-12);
 		    else
-			fprintf(esSpiceF, "%gp", parmval * scale * scale
-				* esScale * esScale);
+			esSIvalue(esSpiceF, 1.0E-12 * (parmval + plist->parm_offset)
+				* scale * scale * esScale * esScale);
 		}
 		else
 		{
@@ -194,15 +194,15 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 				'p' && plist->parm_next->parm_type[1] ==
 				plist->parm_type[1])
 		    {
-			spcnAP(dnode, resclass, scale, plist->parm_name,
-				plist->parm_next->parm_name, sdM,
-				esSpiceF, w);
+			spcnAP(&dev->dev_terms[pn], dnode, resclass, scale,
+				plist->parm_name, plist->parm_next->parm_name,
+				sdM, esSpiceF, w);
 			plist = plist->parm_next;
 		    }
 		    else
 		    {
-			spcnAP(dnode, resclass, scale, plist->parm_name, NULL, sdM,
-				esSpiceF, w);
+			spcnAP(&dev->dev_terms[pn], dnode, resclass, scale,
+				plist->parm_name, NULL, sdM, esSpiceF, w);
 		    }
 		}
 
@@ -220,7 +220,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 			fprintf(esSpiceF, "%g", parmval * scale
 				* esScale * plist->parm_scale * 1E-6);
 		    else
-			fprintf(esSpiceF, "%gu", parmval * scale * esScale);
+			esSIvalue(esSpiceF, (parmval + plist->parm_offset)
+				* scale * esScale * 1.0E-6);
 		}
 		else
 		{
@@ -240,14 +241,15 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 				'a' && plist->parm_next->parm_type[1] ==
 				plist->parm_type[1])
 		    {
-			spcnAP(dnode, resclass, scale, plist->parm_next->parm_name,
+			spcnAP(&dev->dev_terms[pn], dnode, resclass, scale,
+				plist->parm_next->parm_name,
 				plist->parm_name, sdM, esSpiceF, w);
 			plist = plist->parm_next;
 		    }
 		    else
 		    {
-			spcnAP(dnode, resclass, scale, NULL, plist->parm_name, sdM,
-				esSpiceF, w);
+			spcnAP(&dev->dev_terms[pn], dnode, resclass, scale, NULL,
+				plist->parm_name, sdM, esSpiceF, w);
 		    }
 		}
 
@@ -263,7 +265,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 			fprintf(esSpiceF, "%g", l * scale * esScale
 				* plist->parm_scale * 1E-6);
 		    else
-			fprintf(esSpiceF, "%gu", l * scale * esScale);
+			esSIvalue(esSpiceF, 1.0E-6 * (l + plist->parm_offset)
+				* scale * esScale);
 		}
 		else
 		{
@@ -286,7 +289,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 				    fprintf(esSpiceF, "%g", dval * scale * esScale
 						* plist->parm_scale * 1E-6);
 				else
-				    fprintf(esSpiceF, "%gu", dval * scale * esScale);
+				    esSIvalue(esSpiceF, (dval + plist->parm_offset)
+						* scale * esScale * 1.0E-6);
 				dparam->parm_name[0] = '\0';
 				break;
 			    }
@@ -303,7 +307,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 		    fprintf(esSpiceF, "%g", w * scale * esScale
 				* plist->parm_scale * 1E-6);
 		else
-		    fprintf(esSpiceF, "%gu", w * scale * esScale);
+		    esSIvalue(esSpiceF, 1.0E-6 * (w + plist->parm_offset)
+				* scale * esScale);
 		break;
 	    case 's':
 		fprintf(esSpiceF, " %s=", plist->parm_name);
@@ -319,8 +324,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 		    fprintf(esSpiceF, "%g", dev->dev_rect.r_xbot * scale
 				* esScale * plist->parm_scale * 1E-6);
 		else
-		    fprintf(esSpiceF, "%gu", dev->dev_rect.r_xbot * scale
-				* esScale);
+		    esSIvalue(esSpiceF, (dev->dev_rect.r_xbot + plist->parm_offset)
+				* scale * esScale * 1.0E-6);
 		break;
 	    case 'y':
 		fprintf(esSpiceF, " %s=", plist->parm_name);
@@ -330,8 +335,8 @@ spcHierWriteParams(hc, dev, scale, l, w, sdM)
 		    fprintf(esSpiceF, "%g", dev->dev_rect.r_ybot * scale
 				* esScale * plist->parm_scale * 1E-6);
 		else
-		    fprintf(esSpiceF, "%gu", dev->dev_rect.r_ybot * scale
-				* esScale);
+		    esSIvalue(esSpiceF, (dev->dev_rect.r_ybot + plist->parm_offset)
+				* scale * esScale * 1.0E-6);
 		break;
 	    case 'r':
 		fprintf(esSpiceF, " %s=", plist->parm_name);
@@ -426,9 +431,10 @@ esOutputHierResistor(hc, dev, scale, term1, term2, has_model, l, w, dscale)
 	}
 	else
 	{
-	    fprintf(esSpiceF, " w=%gu l=%gu",
-		(float)w * scale * esScale,
-		(float)((l * scale * esScale) / dscale));
+	    fprintf(esSpiceF, " w=");
+	    esSIvalue(esSpiceF, 1.0E-6 * (float)w * scale * esScale);
+	    fprintf(esSpiceF, " l=");
+	    esSIvalue(esSpiceF, 1.0E-6 * (float)((l * scale * esScale) / dscale));
 	}
 	spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	if (sdM != 1.0)
@@ -902,7 +908,7 @@ spcdevHierVisit(hc, dev, scale)
 
 	    /* Capacitor is "Cnnn top bottom value"	*/
 	    /* extraction sets top=gate bottom=source	*/
-	    /* extracted units are fF; output is in fF */
+	    /* extracted units are fF.			*/
 
 	    spcdevOutNode(hc->hc_hierName,
 			gate->dterm_node->efnode_name->efnn_hier,
@@ -920,8 +926,7 @@ spcdevHierVisit(hc, dev, scale)
 
 	    if (!has_model)
 	    {
-		fprintf(esSpiceF, " %ffF", (double)sdM *
-				(double)(dev->dev_cap));
+		esSIvalue(esSpiceF, 1.0E-15 * (double)sdM * (double)dev->dev_cap);
 		spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    }
 	    else
@@ -934,9 +939,10 @@ spcdevHierVisit(hc, dev, scale)
 		}
 		else
 		{
-		    fprintf(esSpiceF, " w=%gu l=%gu",
-			w * scale * esScale,
-			l * scale * esScale);
+		    fprintf(esSpiceF, " w=");
+		    esSIvalue(esSpiceF, 1.0E-6 * w * scale * esScale);
+		    fprintf(esSpiceF, " l=");
+		    esSIvalue(esSpiceF, 1.0E-6 * l * scale * esScale);
 		}
 		spcHierWriteParams(hc, dev, scale, l, w, sdM);
 		if (sdM != 1.0)
@@ -949,7 +955,7 @@ spcdevHierVisit(hc, dev, scale)
 
 	    /* Capacitor is "Cnnn bottom top value"	*/
 	    /* extraction sets top=source bottom=gate	*/
-	    /* extracted units are fF; output is in fF */
+	    /* extracted units are fF.			*/
 
 	    spcdevOutNode(hc->hc_hierName,
 			gate->dterm_node->efnode_name->efnn_hier,
@@ -967,8 +973,7 @@ spcdevHierVisit(hc, dev, scale)
 
 	    if (!has_model)
 	    {
-		fprintf(esSpiceF, " %ffF", (double)sdM *
-				(double)(dev->dev_cap));
+		esSIvalue(esSpiceF, 1.0E-15 * (double)sdM * (double)dev->dev_cap);
 		spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    }
 	    else
@@ -981,9 +986,10 @@ spcdevHierVisit(hc, dev, scale)
 		}
 		else
 		{
-		    fprintf(esSpiceF, " w=%gu l=%gu",
-			w * scale * esScale,
-			l * scale * esScale);
+		    fprintf(esSpiceF, " w=");
+		    esSIvalue(esSpiceF, 1.0E-6 * w * scale * esScale);
+		    fprintf(esSpiceF, " l=");
+		    esSIvalue(esSpiceF, 1.0E-6 * l * scale * esScale);
 		}
 		spcHierWriteParams(hc, dev, scale, l, w, sdM);
 		if (sdM != 1.0)
@@ -1024,9 +1030,10 @@ spcdevHierVisit(hc, dev, scale)
 	    }
 	    else
 	    {
-		fprintf(esSpiceF, " w=%gu l=%gu",
-			w * scale * esScale,
-			l * scale * esScale);
+		fprintf(esSpiceF, " w=");
+		esSIvalue(esSpiceF, 1.0E-6 * w * scale * esScale);
+		fprintf(esSpiceF, " l=");
+		esSIvalue(esSpiceF, 1.0E-6 * l * scale * esScale);
 	    }
 	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    if (sdM != 1.0)
@@ -1042,10 +1049,10 @@ spcdevHierVisit(hc, dev, scale)
 
 	    fprintf(esSpiceF, "\n+ ");
 	    dnode = GetHierNode(hc, drain->dterm_node->efnode_name->efnn_hier);
-            spcnAP(dnode, esFetInfo[dev->dev_type].resClassDrain, scale,
+            spcnAP(drain, dnode, esFetInfo[dev->dev_type].resClassDrain, scale,
 			"ad", "pd", sdM, esSpiceF, w);
 	    snode= GetHierNode(hc, source->dterm_node->efnode_name->efnn_hier);
-	    spcnAP(snode, esFetInfo[dev->dev_type].resClassSource, scale,
+	    spcnAP(source, snode, esFetInfo[dev->dev_type].resClassSource, scale,
 			"as", "ps", sdM, esSpiceF, w);
 	    if (subAP)
 	    {
@@ -1057,8 +1064,8 @@ spcdevHierVisit(hc, dev, scale)
 		    fprintf(esSpiceF, "asub=0 psub=0");
 		}
 		else if (subnodeFlat)
-		    spcnAP(subnodeFlat, esFetInfo[dev->dev_type].resClassSub, scale,
-	       			"asub", "psub", sdM, esSpiceF, -1);
+		    spcnAP(NULL, subnodeFlat, esFetInfo[dev->dev_type].resClassSub,
+				scale, "asub", "psub", sdM, esSpiceF, -1);
 		else
 		    fprintf(esSpiceF, "asub=0 psub=0");
 	    }
@@ -1073,13 +1080,21 @@ spcdevHierVisit(hc, dev, scale)
         case DEV_MSUBCKT:
 	        if (!esNoAttrs)
 	        {
-		    if (gate->dterm_attrs || source->dterm_attrs || drain->dterm_attrs)
+		    bool haveSattr = FALSE;
+		    bool haveDattr = FALSE;
+
+		    if (source->dterm_attrs && (*source->dterm_attrs))
+			haveSattr = TRUE;
+		    if (drain->dterm_attrs && (*drain->dterm_attrs))
+			haveDattr = TRUE;
+
+		    if (gate->dterm_attrs || haveSattr || haveDattr)
 		        fprintf(esSpiceF,"\n**devattr");
 		    if (gate->dterm_attrs)
 		        fprintf(esSpiceF, " g=%s", gate->dterm_attrs);
-		    if (source->dterm_attrs)
+		    if (haveSattr)
 		        fprintf(esSpiceF, " s=%s", source->dterm_attrs);
-		    if (drain->dterm_attrs)
+		    if (haveDattr)
 		        fprintf(esSpiceF, " d=%s", drain->dterm_attrs);
 	        }
 	        break;
@@ -1135,33 +1150,49 @@ spcdevHierMergeVisit(hc, dev, scale)
 	{
 	    /* To-do:  add back source, drain attribute check */
 
+	    /* Default case:  Add the counts together */
+	    m = esFMult[cfp->esFMIndex] + esFMult[fp->esFMIndex];
+
 	    switch(dev->dev_class)
 	    {
 		case DEV_MOSFET:
 		case DEV_MSUBCKT:
 		case DEV_ASYMMETRIC:
 		case DEV_FET:
-		    m = esFMult[cfp->esFMIndex] + (fp->w / cfp->w);
+		    if (cfp->w > 0)
+			m = esFMult[cfp->esFMIndex] + (fp->w / cfp->w);
 		    break;
 		case DEV_RSUBCKT:
 		case DEV_RES:
 		    if ((fp->dev->dev_type == esNoModelType) ||
 			    !strcmp(EFDevTypes[fp->dev->dev_type], "None"))
-			m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
-				/ cfp->dev->dev_res);
+		    {
+			if (cfp->dev->dev_res > 0)
+			    m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
+					/ cfp->dev->dev_res);
+		    }
 		    else
-			m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
+		    {
+			if (cfp->l > 0)
+			    m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
+		    }
 		    break;
 		case DEV_CSUBCKT:
 		case DEV_CAP:
 		case DEV_CAPREV:
 		    if ((fp->dev->dev_type == esNoModelType) ||
 			    !strcmp(EFDevTypes[fp->dev->dev_type], "None"))
-			m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
-				/ cfp->dev->dev_cap);
+		    {
+			if (cfp->dev->dev_cap > 0)
+			    m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
+					/ cfp->dev->dev_cap);
+		    }
 		    else
-			m = esFMult[cfp->esFMIndex] +
-				((fp->l * fp->w) / (cfp->l * cfp->w));
+		    {
+			if ((cfp->l > 0) && (cfp->w > 0))
+			    m = esFMult[cfp->esFMIndex] +
+					((fp->l * fp->w) / (cfp->l * cfp->w));
+		    }
 		    break;
 	    }
 	    setDevMult(fp->esFMIndex, DEV_KILLED);
@@ -1215,9 +1246,11 @@ spccapHierVisit(hc, hierName1, hierName2, cap)
     if (fabs(cap) <= EFCapThreshold)
 	return 0;
 
-    fprintf(esSpiceF, esSpiceCapFormat, esCapNum++,
+    fprintf(esSpiceF, "C%d %s %s ", esCapNum++,
 		nodeSpiceHierName(hc, hierName1),
-                nodeSpiceHierName(hc, hierName2), cap);
+                nodeSpiceHierName(hc, hierName2));
+    esSIvalue(esSpiceF, 1.0E-15 *cap);
+    fprintf(esSpiceF, "\n");
     return 0;
 }
 
@@ -1253,10 +1286,44 @@ spcresistHierVisit(hc, hierName1, hierName2, res)
     HierName *hierName2;
     float res;
 {
+    HashEntry *he;
+    EFNodeName *nn;
+
     fprintf(esSpiceF, "R%d %s %s %g\n", esResNum++,
 		nodeSpiceHierName(hc, hierName1),
                 nodeSpiceHierName(hc, hierName2), res / 1000.);
 
+    /* Mark nodes as visited so that associated capacitances won't be marked
+     * as "floating".  This is inefficient since nodeSpiceName() already does
+     * a hash lookup of the EFNodeName.  Could be improved, but is not a big
+     * performance issue.
+     */
+    he = EFHNLook(hierName1, (char *)NULL, "nodeName");
+    if (he != NULL)
+    {
+	nn = (EFNodeName *)HashGetValue(he);
+
+	/* Mark node as visited (set bit one higher than number of resist classes) */
+	if (esDistrJunct)
+	    update_w(efNumResistClasses, 1, nn->efnn_node);
+	else
+	    markVisited((nodeClientHier *)nn->efnn_node->efnode_client,
+			efNumResistClasses);
+    }
+
+    he = EFHNLook(hierName2, (char *)NULL, "nodeName");
+    if (he != NULL)
+    {
+	nn = (EFNodeName *)HashGetValue(he);
+
+	/* Mark node as visited (set bit one higher than number of resist classes) */
+	if (esDistrJunct)
+	    update_w(efNumResistClasses, 1, nn->efnn_node);
+	else
+	    markVisited((nodeClientHier *)nn->efnn_node->efnode_client,
+			efNumResistClasses);
+    }
+
     return 0;
 }
 
@@ -1352,10 +1419,14 @@ spcnodeHierVisit(hc, node, res, cap)
     cap = cap  / 1000;
     if (fabs(cap) > EFCapThreshold)
     {
-	fprintf(esSpiceF, esSpiceCapFormat, esCapNum++, nsn, cap,
-			(isConnected) ?  "" :
-			(esFormat == NGSPICE) ? " $ **FLOATING" :
-			" **FLOATING");
+	fprintf(esSpiceF, "C%d %s %s ", esCapNum++, nsn, esSpiceCapNode);
+	esSIvalue(esSpiceF, 1.0E-15 * cap);
+	if (!isConnected)
+	{
+	    if (esFormat == NGSPICE) fprintf(esSpiceF, " $");
+	    fprintf(esSpiceF, " **FLOATING");
+	}
+	fprintf(esSpiceF, "\n");
     }
     if (node->efnode_attrs && !esNoAttrs)
     {
@@ -1535,33 +1606,49 @@ devMergeHierVisit(hc, dev, scale)
 		mergeAttr(&cd->dterm_attrs, &drain->dterm_attrs);
 	    }
 mergeThem:
+	    /* Default case:  Add the counts together */
+	    m = esFMult[cfp->esFMIndex] + esFMult[fp->esFMIndex];
+
 	    switch(dev->dev_class)
 	    {
 		case DEV_MOSFET:
 		case DEV_ASYMMETRIC:
 		case DEV_MSUBCKT:
 		case DEV_FET:
-		    m = esFMult[cfp->esFMIndex] + ((float)fp->w / (float)cfp->w);
+		    if (cfp->w > 0)
+			m = esFMult[cfp->esFMIndex] + ((float)fp->w / (float)cfp->w);
 		    break;
 		case DEV_RSUBCKT:
 		case DEV_RES:
 		    if ((fp->dev->dev_type == esNoModelType) ||
 			    !strcmp(EFDevTypes[fp->dev->dev_type], "None"))
-		        m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
-				/ cfp->dev->dev_res);
+		    {
+			if (cfp->dev->dev_res > 0)
+			    m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
+					/ cfp->dev->dev_res);
+		    }
 		    else
-		        m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
+		    {
+			if (cfp->l > 0)
+			    m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
+		    }
 		    break;
 		case DEV_CSUBCKT:
 		case DEV_CAP:
 		case DEV_CAPREV:
 		    if ((fp->dev->dev_type == esNoModelType) ||
 			    !strcmp(EFDevTypes[fp->dev->dev_type], "None"))
-		        m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
-				/ cfp->dev->dev_cap);
+		    {
+			if (cfp->dev->dev_cap > 0)
+			    m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
+					/ cfp->dev->dev_cap);
+		    }
 		    else
-		        m = esFMult[cfp->esFMIndex] +
-				((fp->l  * fp->w) / (cfp->l * cfp->w));
+		    {
+			if ((cfp->l > 0) && (cfp->w > 0))
+			    m = esFMult[cfp->esFMIndex] +
+					((fp->l  * fp->w) / (cfp->l * cfp->w));
+		    }
 		    break;
 	    }
 	    setDevMult(fp->esFMIndex, DEV_KILLED);
@@ -2076,18 +2163,17 @@ esHierVisit(hc, cdata)
 	EFHierVisitResists(hcf, spcresistHierVisit, (ClientData)NULL);
 
 	/* Output coupling capacitances */
-	sprintf( esSpiceCapFormat,  "C%%d %%s %%s %%.%dlffF\n", esCapAccuracy);
 	EFHierVisitCaps(hcf, spccapHierVisit, (ClientData)NULL);
 
 	if (EFCompat == FALSE)
 	{
 	    /* Find the substrate node */
 	    EFHierVisitNodes(hcf, spcsubHierVisit, (ClientData)&resstr);
-	    if (resstr == NULL) resstr = StrDup((char **)NULL, "0");
+	    if (resstr == NULL)
+		resstr = StrDup((char **)NULL, esSpiceDefaultGnd);
 
 	    /* Output lumped capacitance and resistance to substrate */
-	    sprintf( esSpiceCapFormat,  "C%%d %%s %s %%.%dlffF%%s\n",
-			resstr, esCapAccuracy);
+	    esSpiceCapNode = resstr;
 	    EFHierVisitNodes(hcf, spcnodeHierVisit, (ClientData) NULL);
 	    freeMagic(resstr);
 	}
