@@ -179,6 +179,7 @@ typedef struct def
 #define DEF_NODEVICES	0x08	/* This def contains no devices */
 #define DEF_SUBSNODES	0x10	/* This def contains implicit substrate nodes */
 #define DEF_ABSTRACT	0x20	/* This def is an abstract view (e.g., LEF) */
+#define DEF_PRIMITIVE	0x40	/* This def is a primitive device */
 
 /*
  * Every Def has a NULL-terminated list of uses that correspond
@@ -286,6 +287,7 @@ extern Def *EFRootDef();
     /* HierName manipulation */
 extern HierName *efHNFromUse();
 extern char *efHNToStrFunc();
+extern bool EFHNBest();
 
     /* Functions for hashing of HierNames */
 extern int efHNCompare();
