@@ -3,21 +3,21 @@
  *
  * Channel management for the gate-array router.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaChannel.c,v 1.2 2001/01/12 22:12:39 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/garouter/gaChannel.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -42,6 +42,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaChannel.c,v 1.2
 #include "utils/styles.h"
 #include "debug/debug.h"
 
+/* C99 compat */
+#include "garouter/gaInternal.h"
+
 /* List of all active channels */
 GCRChannel *gaChannelList = NULL;
 
@@ -60,7 +63,7 @@ void gaInitRiverBlockages();
 
 int gaTotNormCross, gaTotRiverCross, gaClearNormCross, gaClearRiverCross;
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -86,7 +89,7 @@ GAChannelInitOnce()
     RtrChannelPlane = gaChannelDef->cd_planes[PL_DRC_ERROR];
     GAClearChannels();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -123,7 +126,7 @@ GAClearChannels()
     gaChannelList = (GCRChannel *) NULL;
     SigEnableInterrupts();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -213,7 +216,7 @@ GADefineChannel(chanType, r)
 
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -417,7 +420,7 @@ gaPinStats(pins, nPins, pTot, pClear)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -454,7 +457,7 @@ gaPropagateBlockages(list)
 		changed = TRUE;
     } while (changed);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -480,7 +483,7 @@ gaSetClient(tile, cdata)
     tile->ti_client = (ClientData) cdata;
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -535,7 +538,7 @@ gaSplitTile(tile, r)
     }
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
