@@ -3,7 +3,7 @@
  * Copyright 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
  *
  * This file contains additional functions to manipulate an X window system
- * color display.  Included here are device-dependent routines to draw and 
+ * color display.  Included here are device-dependent routines to draw and
  * erase text and draw a grid.
  *
  */
@@ -11,10 +11,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <GL/gl.h>
-#include <GL/glx.h>
-#include <GL/glu.h>
-
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
 #include "utils/geometry.h"
@@ -22,6 +18,7 @@
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "graphics/graphicsInt.h"
+#include "dbwind/dbwind.h"
 #include "textio/textio.h"
 #include "utils/signals.h"
 #include "utils/utils.h"
@@ -30,13 +27,26 @@
 #include "graphics/grTkCommon.h"
 #include "database/fonts.h"
 
+/* C99 compat
+ * GL headers must be included after graphics/grTOGLInt.h
+ */
+#include <GL/gl.h>
+#include <GL/glx.h>
+#include <GL/glu.h>
+#include <GL/glext.h>
+
 extern Display *grXdpy;
 
 /* locals */
 
 GLuint  grXBases[4];
 
-
+typedef struct {
+   GLuint framebuffer;
+   GLuint renderbuffer;
+} RenderFrame;
+
+
 /*---------------------------------------------------------
  * grtoglDrawGrid:
  *	grxDrawGrid adds a grid to the grid layer, using the current
@@ -70,12 +80,12 @@ grtoglDrawGrid (prect, outline, clip)
     ysize = prect->r_ytop - prect->r_ybot;
     if (!xsize || !ysize || GRID_TOO_SMALL(xsize, ysize))
 	return FALSE;
-    
+
     xstart = prect->r_xbot % xsize;
     while (xstart < clip->r_xbot << SUBPIXELBITS) xstart += xsize;
     ystart = prect->r_ybot % ysize;
     while (ystart < clip->r_ybot << SUBPIXELBITS) ystart += ysize;
-    
+
     grtoglSetLineStyle(outline);
 
     glBegin(GL_LINES);
@@ -105,7 +115,7 @@ grtoglDrawGrid (prect, outline, clip)
     return TRUE;
 }
 
-
+
 /*---------------------------------------------------------
  * grtoglLoadFont
  *	This local routine transfers the X font bitmaps
@@ -137,7 +147,7 @@ grtoglLoadFont()
     return TRUE;
 }
 
-
+
 /*---------------------------------------------------------
  * grtoglSetCharSize:
  *	This local routine sets the character size in the display,
@@ -176,12 +186,12 @@ grtoglSetCharSize (size)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * GrTOGLTextSize --
  *
- *	Determine the size of a text string. 
+ *	Determine the size of a text string.
  *
  * Results:
  *	None.
@@ -202,7 +212,7 @@ GrTOGLTextSize(text, size, r)
     Tk_FontMetrics overall;
     Tk_Font font;
     int width;
-    
+
     switch (size) {
     case GR_TEXT_DEFAULT:
     case GR_TEXT_SMALL:
@@ -233,7 +243,197 @@ GrTOGLTextSize(text, size, r)
     r->r_xbot = 0;
 }
 
-
+/* OpenGL backing store functions (now removed from the X11-based ones) */
+/* Since we always paint into the front buffer, the back buffer is	*/
+/* always available for backing store.  We need not create or destroy	*/
+/* it.	We just use the w_backingStore location to store whether the	*/
+/* backing store contains valid data or not.				*/
+
+void
+grtoglFreeBackingStore(MagWindow *w)
+{
+   RenderFrame *rf;
+
+   rf = (RenderFrame *)w->w_backingStore;
+   if (rf == NULL) return;
+   glDeleteFramebuffers(1, &rf->framebuffer);
+   glDeleteRenderbuffers(1, &rf->renderbuffer);
+   freeMagic(w->w_backingStore);
+   w->w_backingStore = (ClientData)0;
+}
+
+void
+grtoglCreateBackingStore(MagWindow *w)
+{
+   RenderFrame *rf;
+
+   Tk_Window tkwind = (Tk_Window)w->w_grdata;
+   unsigned int width, height;
+
+   /* ignore all windows other than layout */
+   if (w->w_client != DBWclientID) return;
+
+   /* Deferred */
+   if (tkwind == NULL) return;
+
+   width = w->w_screenArea.r_xtop - w->w_screenArea.r_xbot;
+   height = w->w_screenArea.r_ytop - w->w_screenArea.r_ybot;
+
+   rf = (RenderFrame *)w->w_backingStore;
+   if (rf != (ClientData)NULL) {
+	glDeleteFramebuffers(1, &rf->framebuffer);
+	glDeleteRenderbuffers(1, &rf->renderbuffer);
+   }
+   else {
+        rf = (RenderFrame *)mallocMagic(sizeof(RenderFrame));
+	w->w_backingStore = (ClientData)rf;
+   }
+
+   glGenFramebuffers(1, &rf->framebuffer);
+   glGenRenderbuffers(1, &rf->renderbuffer);
+   glBindRenderbuffer(GL_RENDERBUFFER, rf->renderbuffer);
+   glRenderbufferStorage(GL_RENDERBUFFER, GL_RGB, width, height);
+}
+
+bool
+grtoglGetBackingStore(MagWindow *w, Rect *area)
+{
+    RenderFrame *rf;
+    unsigned int width, height;
+    int xbot, ybot;
+    Rect r;
+
+    if (w->w_backingStore == (ClientData)0) return FALSE;
+
+    GEO_EXPAND(area, 1, &r);
+    GeoClip(&r, &(w->w_screenArea));
+
+    width = r.r_xtop - r.r_xbot;
+    height = r.r_ytop - r.r_ybot;
+
+    xbot = r.r_xbot;
+    ybot = r.r_ybot;
+
+    rf = (RenderFrame *)w->w_backingStore;
+
+    glBindFramebuffer(GL_READ_FRAMEBUFFER, rf->framebuffer);
+    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
+    glFramebufferRenderbuffer(GL_READ_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+		GL_RENDERBUFFER, rf->renderbuffer);
+
+    glDrawBuffer(GL_FRONT);
+    glReadBuffer(GL_COLOR_ATTACHMENT0);
+
+    glBlitFramebuffer(r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop,
+		r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop,
+		GL_COLOR_BUFFER_BIT, GL_LINEAR);
+    glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
+
+    return TRUE;
+}
+
+
+bool
+grtoglScrollBackingStore(MagWindow *w, Point *shift)
+{
+    RenderFrame *rf;
+    GLuint FramebufferName, RenderbufferName;
+    unsigned int width, height;
+    int xorigin, yorigin, xshift, yshift;
+
+    if (w->w_backingStore == (ClientData)0)
+    {
+	fprintf(stdout, "grtoglScrollBackingStore %d %d failure\n",
+		shift->p_x, shift->p_y);
+	return FALSE;
+    }
+
+    width = w->w_screenArea.r_xtop - w->w_screenArea.r_xbot;
+    height = w->w_screenArea.r_ytop - w->w_screenArea.r_ybot;
+    xorigin = 0;
+    yorigin = 0;
+    xshift = shift->p_x;
+    yshift = shift->p_y;
+
+    if (xshift > 0)
+        width -= xshift;
+    else if (xshift < 0)
+    {
+        width += xshift;
+        xorigin = -xshift;
+        xshift = 0;
+    }
+    if (yshift > 0)
+        height -= yshift;
+    else if (yshift < 0)
+    {
+        height += yshift;
+        yorigin = -yshift;
+        yshift = 0;
+    }
+
+    rf = (RenderFrame *)w->w_backingStore;
+
+    glBindFramebuffer(GL_READ_FRAMEBUFFER, rf->framebuffer);
+    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
+    glFramebufferRenderbuffer(GL_READ_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+		GL_RENDERBUFFER, rf->renderbuffer);
+
+    glBlitFramebuffer(xorigin, yorigin, xorigin + width, yorigin + height,
+		xshift, yshift, xshift + width, yshift + height,
+		GL_COLOR_BUFFER_BIT, GL_LINEAR);
+
+    glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
+    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, rf->framebuffer);
+    glBlitFramebuffer(xshift, yshift, xshift + width, yshift + height,
+		xshift, yshift, xshift + width, yshift + height,
+		GL_COLOR_BUFFER_BIT, GL_LINEAR);
+    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
+
+    return TRUE;
+}
+
+void
+grtoglPutBackingStore(MagWindow *w, Rect *area)
+{
+    RenderFrame *rf;
+    GLuint FramebufferName, RenderbufferName;
+    unsigned int width, height;
+    int ybot, xbot;
+    Rect r;
+
+    if (w->w_backingStore == (ClientData)0) return;
+
+    if (w->w_flags & WIND_OBSCURED)
+    {
+	grtoglFreeBackingStore(w);
+	return;
+    }
+
+    GEO_EXPAND(area, 1, &r);
+    GeoClip(&r, &(w->w_screenArea));
+
+    width = r.r_xtop - r.r_xbot;
+    height = r.r_ytop - r.r_ybot;
+    ybot = r.r_ybot;
+    xbot = r.r_xbot;
+
+    rf = (RenderFrame *)w->w_backingStore;
+    glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
+    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, rf->framebuffer);
+    glFramebufferRenderbuffer(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+		GL_RENDERBUFFER, rf->renderbuffer);
+
+    glDrawBuffer(GL_COLOR_ATTACHMENT0);
+    glReadBuffer(GL_FRONT);
+
+    glBlitFramebuffer(r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop,
+		r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop,
+		GL_COLOR_BUFFER_BIT, GL_LINEAR);
+    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  * GrTOGLReadPixel --
@@ -257,7 +457,7 @@ GrTOGLReadPixel (w, x, y)
     return 0;		/* OpenGL has no such function, so return 0 */
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * GrTOGLBitBlt --
@@ -302,7 +502,7 @@ myCombine(GLdouble coords[3], GLdouble *vertex_data[4],
     new[1] = coords[1];
     *outData = new;
     /* Diagnostic */
-    TxError("Intersecting polygon in char \"%c\" at %g %g!\n", 
+    TxError("Intersecting polygon in char \"%c\" at %g %g!\n",
 	*((char *)dataptr), coords[0], coords[1]);
 }
 
@@ -389,7 +589,7 @@ grtoglDrawCharacter(clist, tc, pixsize)
  *
  *	For speed, we should be transferring the font
  *	vectors into OpenGL display lists!
- *	
+ *
  *---------------------------------------------------------
  */
 
@@ -401,7 +601,7 @@ grtoglFontText(text, font, size, rotate, pos, clip, obscure)
     int	  rotate;		/* Text rotation */
     Point *pos;			/* Text base position */
     Rect  *clip;		/* Clipping area */
-    LinkedRect *obscure;	/* List of obscuring areas */ 
+    LinkedRect *obscure;	/* List of obscuring areas */
 {
     char *tptr;
     Point *coffset;		/* vector to next character */
@@ -454,11 +654,11 @@ grtoglFontText(text, font, size, rotate, pos, clip, obscure)
  *	the screen -- no clipping is done except to the obscuring rectangle
  *	list and the clip rectangle.
  *
- * Results:	
+ * Results:
  *	none.
  *
  * Side Effects:
- *	The text is drawn on the screen.  
+ *	The text is drawn on the screen.
  *
  *---------------------------------------------------------
  */
@@ -498,7 +698,7 @@ grtoglPutText (text, pos, clip, obscure)
 	    grTOGLGeoSub(&location, &overlap);
 	}
     }
- 
+
     overlap = location;
     GeoClip(&overlap, clip);
 
